import configparser
from typing import Any

from djmix import utils

__all__ = [
  'get_root',
  'set_root',
]

_config_path = utils.mkpath('~/.djmix.ini')
_config = configparser.ConfigParser()
_config.read(_config_path)
if not _config.has_section('djmix'):
  _config.add_section('djmix')
_section = _config['djmix']


def get_root():
  return utils.mkpath(_section.get('root', '~/djmix'))


def set_root(path: str):
  _set('root', path)


def _set(key: str, value: Any):
  _section[key] = value
  _save()


def _save():
  with open(_config_path, 'w') as f:
    _config.write(f)



import os
import abc
from pydantic import BaseModel
from typing import Optional
from djmix import config, utils


class Audio(BaseModel, abc.ABC):
  path: Optional[str]
  
  def exists(self):
    if self.path is None:
      return False
    
    if os.path.isfile(self.path):
      return True
    
    return False
  
  @abc.abstractmethod
  def download(self):
    pass



import json
import logging
from pathlib import Path
from djmix import config
from .models import Mix


def bootstrap():
  logging.basicConfig(level=logging.INFO)
  
  package_path = Path(__file__).parent.parent.parent.resolve()
  metadata_path = package_path.joinpath('dataset/djmix-dataset.json')
  assert metadata_path.is_file(), f'djmix-dataset.json does not exist at: {metadata_path}'
  
  json_mixes = json.load(open(metadata_path))
  
  data_dir = config.get_root()
  mixes = [
    Mix(data_dir, **json_mix)
    for json_mix in json_mixes
  ]
  
  return mixes



import logging
import traceback
from djmix import utils
from .Audio import Audio
from .Track import Track
from .download import download_mix
from typing import List


class Mix(Audio):
  id: str
  title: str
  url: str
  audio_source: str
  audio_url: str
  num_identified_tracks: int
  num_total_tracks: int
  num_available_transitions: int
  num_timestamps: int
  tracklist: List[Track]
  tags: List
  
  def __init__(self, data_dir: str, **data):
    mix_id = data['id']
    data['path'] = utils.mkpath(data_dir, 'mixes', f'{mix_id}.mp3')
    
    tracks = []
    for track in data['tracklist']:
      track_id = track['id']
      if track_id is None:
        track['path'] = None
      else:
        track['path'] = utils.mkpath(data_dir, 'tracks', track_id[0], f'{track_id}.mp3')
      tracks.append(Track(**track))
    data['tracklist'] = tracks
    
    super().__init__(**data)
  
  def __repr__(self):
    return f'Mix(id={self.id}, title="{self.title}")'
  
  def download(self):
    download_mix(self)
    
    for track in self.tracklist:
      try:
        if track.id is None:
          continue
        track.download()
      except Exception as e:
        logging.error(f'Failed to download track: https://www.youtube.com/watch?v={track["id"]}')
        traceback.print_exc()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['djmix', 'djmix.models']

package_data = \
{'': ['*']}

install_requires = \
['cvxpy>=1.2.1,<2.0.0',
 'joblib>=1.1.0,<2.0.0',
 'librosa>=0.9.2,<0.10.0',
 'madmom>=0.16.1,<0.17.0',
 'pandas>=1.4.3,<2.0.0',
 'pydantic>=1.9.2,<2.0.0',
 'pydub>=0.25.1,<0.26.0',
 'pytsmod>=0.3.5,<0.4.0',
 'scikit-image>=0.19.3,<0.20.0',
 'yt-dlp']

setup_kwargs = {
    'name': 'djmix',
    'version': '0.1.1',
    'description': '',
    'long_description': None,
    'author': 'Taejun Kim',
    'author_email': 'taejun@kaist.ac.kr',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<3.11',
}


setup(**setup_kwargs)



from .Audio import Audio
from .download import download_track
from typing import Optional


class Track(Audio):
  id: Optional[str]
  title: str
  
  def __repr__(self):
    return f'Track(id={self.id}, title="{self.title}")'
  
  def download(self):
    download_track(self)



from __future__ import annotations

import os
import logging
from typing import TYPE_CHECKING
from yt_dlp import YoutubeDL
from djmix import config, utils

if TYPE_CHECKING:
  from djmix.models import Mix, Track


def download_mix(mix: Mix):
  logging.info(f'=> Start downloading {mix.id}')
  root = config.get_root()
  download_audio(
    url=mix.audio_url,
    path=utils.mkpath(root, 'mixes', f'{mix.id}.mp3')
  )


def download_track(track: Track):
  url = f'https://www.youtube.com/watch?v={track.id}'
  logging.info(f'=> Start downloading track {url}')
  root = config.get_root()
  track_path = utils.mkpath(root, 'tracks', track.id[0], f'{track.id}.mp3')
  download_audio(
    url=url,
    path=track_path,
  )


def download_audio(url, path):
  if os.path.isfile(path):
    logging.info(f'{path} already exists. Skip downloading.')
    return
  
  params = {
    'format': 'bestaudio',
    'outtmpl': path,
    'postprocessors': [{  # Extract audio using ffmpeg
      'key': 'FFmpegExtractAudio',
      'preferredcodec': 'mp3',
    }]
  }
  with YoutubeDL(params) as ydl:
    os.makedirs(os.path.dirname(path), exist_ok=True)
    ydl.download(url)
